@@ -1,14 +1,14 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __CONFIG_H_ENV_
-#define __CONFIG_H_ENV_ 1
+#ifndef OPENAFS_WINNT_AFSD_CONFIG_H
+#define OPENAFS_WINNT_AFSD_CONFIG_H 1
 
 #define CM_CONFIGDEFAULT_CACHESIZE	98304
 #define CM_CONFIGDEFAULT_BLOCKSIZE	4096
@@ -28,7 +28,7 @@ typedef FILE cm_configFile_t;
 
 typedef long (cm_configProc_t)(void *rockp, struct sockaddr_in *addrp, char *namep, unsigned short);
 
-typedef long (cm_enumCellRegistryProc_t)(void *rockp, char *cellNamep);
+typedef long (cm_enumCellProc_t)(void *rockp, char *cellNamep);
 
 extern long cm_GetRootCellName(char *namep);
 
@@ -39,13 +39,17 @@ extern long cm_SearchCellFileEx(char *cellNamep, char *newCellNamep,
                                 char *linkedNamep,
                                 cm_configProc_t *procp, void *rockp);
 
-extern long cm_SearchCellRegistry(afs_uint32 client, 
+extern long cm_EnumerateCellFile(afs_uint32 client,
+                                 cm_enumCellProc_t *procp,
+                                 void *rockp);
+
+extern long cm_SearchCellRegistry(afs_uint32 client,
                                   char *cellNamep, char *newCellNamep,
                                   char *linkedNamep,
                                   cm_configProc_t *procp, void *rockp);
 
-extern long cm_EnumerateCellRegistry(afs_uint32 client, 
-                                     cm_enumCellRegistryProc_t *procp, 
+extern long cm_EnumerateCellRegistry(afs_uint32 client,
+                                     cm_enumCellProc_t *procp,
                                      void *rockp);
 
 extern long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
@@ -65,6 +69,13 @@ extern long cm_AppendNewCellLine(cm_configFile_t *filep, char *linep);
 
 extern long cm_CloseCellFile(cm_configFile_t *filep);
 
+extern long cm_AddCellToRegistry( char * cellname,
+                                  char * linked_cellname,
+                                  unsigned short vlport,
+                                  afs_uint32 host_count,
+                                  char *hostname[],
+                                  afs_uint32 flags);
+
 extern long cm_GetCellServDB(char *cellNamep, afs_uint32 len);
 
 extern void cm_GetConfigDir(char *dir, afs_uint32 len);
@@ -76,4 +87,4 @@ extern void cm_GetConfigDir(char *dir, afs_uint32 len);
 
 #endif /* __CM_CONFIG_INTERFACES_ONLY__ */
 
-#endif /* __CONFIG_H_ENV_ */
+#endif /* OPENAFS_WINNT_AFSD_CONFIG_H */
