@@ -14,6 +14,7 @@
 #include <winsock2.h>
 #include <nb30.h>
 #include <string.h>
+#include <strsafe.h>
 #include <malloc.h>
 #include "afsd.h"
 #include <osi.h>
@@ -104,6 +105,7 @@ void cm_InitVolume(int newFile, long maxVols)
                 lock_InitializeRWLock(&volp->rw, "cm_volume_t rwlock", LOCK_HIERARCHY_VOLUME);
                 _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_RESET);
                 _InterlockedAnd(&volp->flags, ~CM_VOLUMEFLAG_UPDATING_VL);
+                volp->lastUpdateTime = 0;
                 for (volType = RWVOL; volType < NUM_VOL_TYPES; volType++) {
                     volp->vol[volType].state = vl_unknown;
                     volp->vol[volType].serversp = NULL;
@@ -160,10 +162,76 @@ cm_VolNameIsID(char *aname)
  *    first, and fall back to successively older versions if you get
  *    RXGEN_OPCODE.
  */
-#define MULTIHOMED 1
+static long
+cm_GetEntryByName( struct cm_cell *cellp, const char *name,
+                   struct vldbentry *vldbEntryp,
+                   struct nvldbentry *nvldbEntryp,
+                   struct uvldbentry *uvldbEntryp,
+                   int *methodp,
+                   cm_user_t *userp,
+                   cm_req_t *reqp
+                   )
+{
+    long code;
+    cm_conn_t *connp;
+    struct rx_connection * rxconnp;
+
+    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s",
+              osi_LogSaveString(afsd_logp,cellp->name),
+              osi_LogSaveString(afsd_logp,name));
+    do {
+
+        code = cm_ConnByMServers(cellp->vlServersp, FALSE, userp, reqp, &connp);
+        if (code)
+            continue;
+
+        rxconnp = cm_GetRxConn(connp);
+        code = VL_GetEntryByNameU(rxconnp, name, uvldbEntryp);
+        *methodp = 2;
+        if ( code == RXGEN_OPCODE )
+        {
+            code = VL_GetEntryByNameN(rxconnp, name, nvldbEntryp);
+            *methodp = 1;
+        }
+        if ( code == RXGEN_OPCODE ) {
+            code = VL_GetEntryByNameO(rxconnp, name, vldbEntryp);
+            *methodp = 0;
+        }
+        rx_PutConnection(rxconnp);
+    } while (cm_Analyze(connp, userp, reqp, NULL, 0, NULL, cellp->vlServersp, NULL, code));
+    code = cm_MapVLRPCError(code, reqp);
+    if ( code )
+        osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x",
+                  osi_LogSaveString(afsd_logp,cellp->name),
+                  osi_LogSaveString(afsd_logp,name), code);
+    else
+        osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS",
+                  osi_LogSaveString(afsd_logp,cellp->name),
+                  osi_LogSaveString(afsd_logp,name));
+    return code;
+}
+
+static long
+cm_GetEntryByID( struct cm_cell *cellp, afs_uint32 id,
+                 struct vldbentry *vldbEntryp,
+                 struct nvldbentry *nvldbEntryp,
+                 struct uvldbentry *uvldbEntryp,
+                 int *methodp,
+                 cm_user_t *userp,
+                 cm_req_t *reqp
+                 )
+{
+    char name[64];
+
+    StringCbPrintf(name, sizeof(name), "%u", id);
+
+    return cm_GetEntryByName(cellp, name, vldbEntryp, nvldbEntryp, uvldbEntryp, methodp, userp, reqp);
+}
+
 long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *reqp,
 		     cm_volume_t *volp)
 {
+    struct rx_connection *rxconnp;
     cm_conn_t *connp;
     int i;
     afs_uint32 j, k;
@@ -174,9 +242,7 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
     u_long tempAddr;
     struct vldbentry vldbEntry;
     struct nvldbentry nvldbEntry;
-#ifdef MULTIHOMED
     struct uvldbentry uvldbEntry;
-#endif
     int method = -1;
     int ROcount = 0;
     long code;
@@ -188,6 +254,7 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
 #endif
     afs_uint32 volType;
     time_t now;
+    int replicated = 0;
 
     lock_AssertWrite(&volp->rw);
 
@@ -228,6 +295,15 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                 osi_Wakeup((LONG_PTR) &volp->flags);
                 return 0;
             }
+            now = time(NULL);
+        }
+
+        /* Do not query again if the last update attempt failed in the last 60 seconds */
+        if ((volp->flags & CM_VOLUMEFLAG_RESET) && (volp->lastUpdateTime > now - 60))
+        {
+            osi_Log3(afsd_logp, "cm_UpdateVolumeLocation unsuccessful update in last 60 seconds -- name %s:%s flags 0x%x",
+                      volp->cellp->name, volp->namep, volp->flags);
+            return(CM_ERROR_ALLDOWN);
         }
 
         _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_UPDATING_VL);
@@ -237,41 +313,9 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             cm_UpdateCell(cellp, 0);
 
         /* now we have volume structure locked and held; make RPC to fill it */
-	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s",
-                  osi_LogSaveString(afsd_logp,volp->cellp->name),
-                  osi_LogSaveString(afsd_logp,volp->namep));
-        do {
-            struct rx_connection * rxconnp;
-
-            code = cm_ConnByMServers(cellp->vlServersp, userp, reqp, &connp);
-            if (code)
-                continue;
-
-            rxconnp = cm_GetRxConn(connp);
-#ifdef MULTIHOMED
-            code = VL_GetEntryByNameU(rxconnp, volp->namep, &uvldbEntry);
-            method = 2;
-            if ( code == RXGEN_OPCODE )
-#endif
-            {
-                code = VL_GetEntryByNameN(rxconnp, volp->namep, &nvldbEntry);
-                method = 1;
-            }
-            if ( code == RXGEN_OPCODE ) {
-                code = VL_GetEntryByNameO(rxconnp, volp->namep, &vldbEntry);
-                method = 0;
-            }
-            rx_PutConnection(rxconnp);
-        } while (cm_Analyze(connp, userp, reqp, NULL, NULL, cellp->vlServersp, NULL, code));
-        code = cm_MapVLRPCError(code, reqp);
-	if ( code )
-	    osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x",
-		      osi_LogSaveString(afsd_logp,volp->cellp->name),
-                      osi_LogSaveString(afsd_logp,volp->namep), code);
-	else
-	    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS",
-		      osi_LogSaveString(afsd_logp,volp->cellp->name),
-                      osi_LogSaveString(afsd_logp,volp->namep));
+        code = cm_GetEntryByName(cellp, volp->namep, &vldbEntry, &nvldbEntry,
+                                 &uvldbEntry,
+                                 &method, userp, reqp);
     }
 
     /* We can end up here with code == CM_ERROR_NOSUCHVOLUME if the base volume name
@@ -288,41 +332,26 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         snprintf(name, VL_MAXNAMELEN, "%s.readonly", volp->namep);
 
         /* now we have volume structure locked and held; make RPC to fill it */
-	osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s",
-                 osi_LogSaveString(afsd_logp,volp->cellp->name),
-                 osi_LogSaveString(afsd_logp,name));
-        do {
-            struct rx_connection * rxconnp;
+        code = cm_GetEntryByName(cellp, name, &vldbEntry, &nvldbEntry,
+                                 &uvldbEntry,
+                                 &method, userp, reqp);
+    }
 
-            code = cm_ConnByMServers(cellp->vlServersp, userp, reqp, &connp);
-            if (code)
-                continue;
-
-            rxconnp = cm_GetRxConn(connp);
-#ifdef MULTIHOMED
-            code = VL_GetEntryByNameU(connp->rxconnp, name, &uvldbEntry);
-            method = 2;
-            if ( code == RXGEN_OPCODE )
-#endif
-            {
-                code = VL_GetEntryByNameN(connp->rxconnp, name, &nvldbEntry);
-                method = 1;
-            }
-            if ( code == RXGEN_OPCODE ) {
-                code = VL_GetEntryByNameO(connp->rxconnp, name, &vldbEntry);
-                method = 0;
-            }
-            rx_PutConnection(rxconnp);
-        } while (cm_Analyze(connp, userp, reqp, NULL, NULL, cellp->vlServersp, NULL, code));
-        code = cm_MapVLRPCError(code, reqp);
-	if ( code )
-	    osi_Log3(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s FAILURE, code 0x%x",
-		     osi_LogSaveString(afsd_logp,volp->cellp->name),
-                     osi_LogSaveString(afsd_logp,name), code);
-	else
-	    osi_Log2(afsd_logp, "CALL VL_GetEntryByName{UNO} name %s:%s SUCCESS",
-		     osi_LogSaveString(afsd_logp,volp->cellp->name),
-                     osi_LogSaveString(afsd_logp,name));
+    /*
+     * What if there was a volume rename?  The volume name no longer exists but the
+     * volume id might.  Try to refresh the volume location information based one
+     * of the readwrite or readonly volume id.
+     */
+    if (code == CM_ERROR_NOSUCHVOLUME) {
+        if (volp->vol[RWVOL].ID != 0) {
+            code = cm_GetEntryByID(cellp, volp->vol[RWVOL].ID, &vldbEntry, &nvldbEntry,
+                                    &uvldbEntry,
+                                    &method, userp, reqp);
+        } else if (volp->vol[ROVOL].ID != 0) {
+            code = cm_GetEntryByID(cellp, volp->vol[ROVOL].ID, &vldbEntry, &nvldbEntry,
+                                    &uvldbEntry,
+                                    &method, userp, reqp);
+        }
     }
 
     lock_ObtainWrite(&volp->rw);
@@ -357,6 +386,7 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         case 0:
             flags = vldbEntry.flags;
             nServers = vldbEntry.nServers;
+            replicated = (nServers > 0);
             rwID = vldbEntry.volumeId[0];
             roID = vldbEntry.volumeId[1];
             bkID = vldbEntry.volumeId[2];
@@ -370,6 +400,7 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
         case 1:
             flags = nvldbEntry.flags;
             nServers = nvldbEntry.nServers;
+            replicated = (nServers > 0);
             rwID = nvldbEntry.volumeId[0];
             roID = nvldbEntry.volumeId[1];
             bkID = nvldbEntry.volumeId[2];
@@ -380,10 +411,10 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             strncpy(name, nvldbEntry.name, VL_MAXNAMELEN);
             name[VL_MAXNAMELEN - 1] = '\0';
             break;
-#ifdef MULTIHOMED
         case 2:
             flags = uvldbEntry.flags;
             nServers = uvldbEntry.nServers;
+            replicated = (nServers > 0);
             rwID = uvldbEntry.volumeId[0];
             roID = uvldbEntry.volumeId[1];
             bkID = uvldbEntry.volumeId[2];
@@ -405,16 +436,14 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                     memset(&addrs, 0, sizeof(addrs));
 
                     do {
-                        struct rx_connection *rxconnp;
-
-                        code = cm_ConnByMServers(cellp->vlServersp, userp, reqp, &connp);
+                        code = cm_ConnByMServers(cellp->vlServersp, FALSE, userp, reqp, &connp);
                         if (code)
                             continue;
 
                         rxconnp = cm_GetRxConn(connp);
                         code = VL_GetAddrsU(rxconnp, &attrs, &uuid, &unique, &nentries, &addrs);
                         rx_PutConnection(rxconnp);
-                    } while (cm_Analyze(connp, userp, reqp, NULL, NULL, cellp->vlServersp, NULL, code));
+                    } while (cm_Analyze(connp, userp, reqp, NULL, 0, NULL, cellp->vlServersp, NULL, code));
 
                     if ( code ) {
                         code = cm_MapVLRPCError(code, reqp);
@@ -441,12 +470,11 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             strncpy(name, uvldbEntry.name, VL_MAXNAMELEN);
             name[VL_MAXNAMELEN - 1] = '\0';
             break;
-#endif
         }
 
         /* decode the response */
         lock_ObtainWrite(&cm_volumeLock);
-        if (cm_VolNameIsID(volp->namep)) {
+        if (!cm_VolNameIsID(volp->namep)) {
             size_t    len;
 
             len = strlen(name);
@@ -493,6 +521,10 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                 volp->vol[ROVOL].ID = roID;
                 cm_AddVolumeToIDHashTable(volp, ROVOL);
             }
+            if (replicated)
+                _InterlockedOr(&volp->vol[ROVOL].flags, CM_VOL_STATE_FLAG_REPLICATED);
+            else
+                _InterlockedAnd(&volp->vol[ROVOL].flags, ~CM_VOL_STATE_FLAG_REPLICATED);
         } else {
             if (volp->vol[ROVOL].qflags & CM_VOLUME_QFLAG_IN_HASH)
                 cm_RemoveVolumeFromIDHashTable(volp, ROVOL);
@@ -521,7 +553,6 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
             tempAddr = htonl(serverNumber[i]);
             tsockAddr.sin_addr.s_addr = tempAddr;
             tsp = cm_FindServer(&tsockAddr, CM_SERVER_FILE, FALSE);
-#ifdef MULTIHOMED
             if (tsp && (method == 2) && (tsp->flags & CM_SERVERFLAG_UUID)) {
                 /*
                  * Check to see if the uuid of the server we know at this address
@@ -542,7 +573,6 @@ long cm_UpdateVolumeLocation(struct cm_cell *cellp, cm_user_t *userp, cm_req_t *
                               osi_LogSaveString(afsd_logp, hoststr));
                 }
             }
-#endif
             if (!tsp) {
                 /*
                  * cm_NewServer will probe the file server which in turn will
@@ -932,6 +962,7 @@ long cm_FindVolumeByName(struct cm_cell *cellp, char *volumeNamep,
 	strncpy(volp->namep, name, VL_MAXNAMELEN);
 	volp->namep[VL_MAXNAMELEN-1] = '\0';
 	volp->flags = CM_VOLUMEFLAG_RESET;
+        volp->lastUpdateTime = 0;
 
         for ( volType = RWVOL; volType < NUM_VOL_TYPES; volType++) {
             volp->vol[volType].state = vl_unknown;
@@ -1072,6 +1103,7 @@ long cm_ForceUpdateVolume(cm_fid_t *fidp, cm_user_t *userp, cm_req_t *reqp)
     cm_data.mountRootGen = time(NULL);
     lock_ObtainWrite(&volp->rw);
     _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_RESET);
+    volp->lastUpdateTime = 0;
 
     code = cm_UpdateVolumeLocation(cellp, userp, reqp, volp);
     lock_ReleaseWrite(&volp->rw);
@@ -1106,6 +1138,7 @@ cm_serverRef_t **cm_GetVolServers(cm_volume_t *volp, afs_uint32 volume, cm_user_
             firstTry = 0;
             lock_ObtainWrite(&volp->rw);
             _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_RESET);
+            volp->lastUpdateTime = 0;
             code = cm_UpdateVolumeLocation(volp->cellp, userp, reqp, volp);
             lock_ReleaseWrite(&volp->rw);
             if (code == 0)
@@ -1172,8 +1205,10 @@ void cm_RefreshVolumes(int lifetime)
 
         if (!(volp->flags & CM_VOLUMEFLAG_RESET)) {
             lock_ObtainWrite(&volp->rw);
-            if (volp->lastUpdateTime + lifetime <= now)
+            if (volp->lastUpdateTime + lifetime <= now) {
                 _InterlockedOr(&volp->flags, CM_VOLUMEFLAG_RESET);
+                volp->lastUpdateTime = 0;
+            }
             lock_ReleaseWrite(&volp->rw);
         }
 
@@ -1226,8 +1261,9 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
                     continue;
 
                 alldeleted = 0;
-                *onlinep = 1;
-                alldown = 0;
+
+                if (!(serversp->server->flags & CM_SERVERFLAG_DOWN))
+                    alldown = 0;
 
                 if (serversp->status == srv_busy || serversp->status == srv_offline)
                     serversp->status = srv_not_busy;
@@ -1255,7 +1291,7 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
                     code = RXAFS_GetVolumeStatus(rxconnp, statep->ID,
                                                  &volStat, &Name, &OfflineMsg, &MOTD);
                     rx_PutConnection(rxconnp);
-                } while (cm_Analyze(connp, cm_rootUserp, &req, &fid, NULL, NULL, NULL, code));
+                } while (cm_Analyze(connp, cm_rootUserp, &req, &fid, 0, NULL, NULL, NULL, code));
                 code = cm_MapRPCError(code, &req);
 
                 lock_ObtainWrite(&volp->rw);
@@ -1272,10 +1308,12 @@ cm_CheckOfflineVolumeState(cm_volume_t *volp, cm_vol_state_t *statep, afs_uint32
                 cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_alldown);
                 statep->state = vl_alldown;
             }
-        } else if (statep->state != vl_alldown) {
+        } else {
             lock_ReleaseRead(&cm_serverLock);
-            cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_alldown);
-            statep->state = vl_alldown;
+            if (statep->state != vl_alldown) {
+                cm_VolumeStatusNotification(volp, statep->ID, statep->state, vl_alldown);
+                statep->state = vl_alldown;
+            }
         }
     }
 }
