@@ -35,8 +35,6 @@ extern "C" {
 
   typedef BYTE lana_number_t;
 
-#define LANA_INVALID 0xff
-
     struct LANAINFO
     {
         lana_number_t lana_number;
@@ -44,18 +42,20 @@ extern "C" {
     };
 
 #define LANA_INVALID 0xff
-#define MAX_NB_NAME_LENGTH 17
+#define MAX_NB_NAME_LENGTH 16
 
 #define LANA_NETBIOS_NAME_SUFFIX 1
 #define LANA_NETBIOS_NAME_FULL 0
 
 #define LANA_NETBIOS_NAME_IN 2
 
+#define LANA_NETBIOS_NO_RESET 4
+
     int lana_GetNameFromGuid(char *Guid, char **Name);
 
     struct LANAINFO * lana_FindLanaByName(const char *LanaName);
 
-    lana_number_t lana_FindLoopback(void);
+    lana_number_t lana_FindLoopback(BOOL reset);
 
     BOOL lana_OnlyLoopback(void);
 
