@@ -341,9 +341,10 @@ afs_RebuildDynroot(void)
 	c = afs_GetCellByIndex(cellidx, READ_LOCK);
 	if (!c)
 	    break;
-	if (c->cellNum == afs_dynrootCell)
+	if ((c->cellNum == afs_dynrootCell) || (c->states & CHush)) {
+	    afs_PutCell(c, READ_LOCK);
 	    continue;
-
+	}
 	dotLen = strlen(c->cellName) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
 	strcpy(dotCell, ".");
@@ -403,8 +404,7 @@ afs_RebuildDynroot(void)
     dirHeader->alloMap[0] = EPP - DHE - 1;
     for (i = 1; i < MAXPAGES; i++)
 	dirHeader->alloMap[i] = EPP;
-    for (i = 0; i < NHASHENT; i++)
-	dirHeader->hashTable[i] = 0;
+    memset(dirHeader->hashTable, 0, NHASHENT * sizeof(dirHeader->hashTable[0]));
 
     /* Install ".", "..", and the dynamic mount directory */
     afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1);
@@ -417,8 +417,10 @@ afs_RebuildDynroot(void)
 	c = afs_GetCellByIndex(cellidx, READ_LOCK);
 	if (!c)
 	    continue;
-	if (c->cellNum == afs_dynrootCell)
+	if ((c->cellNum == afs_dynrootCell) || (c->states & CHush)) {
+	    afs_PutCell(c, READ_LOCK);
 	    continue;
+	}
 
 	dotLen = strlen(c->cellName) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
@@ -498,8 +500,7 @@ afs_RebuildDynrootMount(void)
     dirHeader->alloMap[0] = EPP - DHE - 1;
     for (i = 1; i < MAXPAGES; i++)
 	dirHeader->alloMap[i] = EPP;
-    for (i = 0; i < NHASHENT; i++)
-	dirHeader->hashTable[i] = 0;
+    memset(dirHeader->hashTable, 0, NHASHENT * sizeof(dirHeader->hashTable[0]));
 
     /* Install "." and ".." */
     afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1);
@@ -603,13 +604,13 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 	    return 0;
 	afs_GetDynroot(0, 0, status);
 	afs_PutDynroot();
-	return 1;
+	goto succeed;
     }
 
     if (afs_IsDynrootMount(avc)) {
 	afs_GetDynrootMount(0, 0, status);
 	afs_PutDynroot();
-	return 1;
+	goto succeed;
     }
 
     /*
@@ -654,7 +655,10 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 	    }
 	    ReleaseReadLock(&afs_dynSymlinkLock);
 
-	    return ts ? 1 : 0;
+	    if (ts)
+		goto succeed;
+
+	    return 0;
 	}
 
 	if (VNUM_TO_VNTYPE(avc->f.fid.Fid.Vnode) != VN_TYPE_CELL
@@ -746,10 +750,31 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 	}
 
 	status->Length = linklen;
-	return 1;
+	goto succeed;
     }
 
     return 0;
+
+    /* make sure we set type correctly when we do this. used to stay VREG */
+ succeed:
+    switch (status->FileType) {
+    case File:
+	vSetType(avc, VREG);
+	break;
+    case Directory:
+	vSetType(avc, VDIR);
+	break;
+    case SymbolicLink:
+	if (afs_fakestat_enable && (avc->f.m.Mode & 0111) == 0)
+	    vSetType(avc, VDIR);
+	else
+	    vSetType(avc, VLNK);
+	break;
+    default:
+	/* shouldn't happen */
+      ;
+    }
+    return 1;
 }
 
 /*
