@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -66,6 +66,13 @@
 #define SYS_NAME_ID_ppc64_darwin_100     513
 #define SYS_NAME_ID_x86_darwin_100       514
 #define SYS_NAME_ID_amd64_darwin_100     515
+#define SYS_NAME_ID_arm_darwin_100       516
+#define SYS_NAME_ID_x86_darwin_110       517
+#define SYS_NAME_ID_amd64_darwin_110     518
+#define SYS_NAME_ID_arm_darwin_110       519
+#define SYS_NAME_ID_x86_darwin_120       520
+#define SYS_NAME_ID_amd64_darwin_120     521
+#define SYS_NAME_ID_arm_darwin_120       522
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -192,7 +199,13 @@
 #define SYS_NAME_ID_i386_fbsd_62        2114
 #define SYS_NAME_ID_i386_fbsd_70        2115
 #define SYS_NAME_ID_i386_fbsd_71        2116
+#define SYS_NAME_ID_i386_fbsd_72        2130
+#define SYS_NAME_ID_i386_fbsd_73        2131
+#define SYS_NAME_ID_i386_fbsd_74        2132
 #define SYS_NAME_ID_i386_fbsd_80        2117
+#define SYS_NAME_ID_i386_fbsd_81        2118
+#define SYS_NAME_ID_i386_fbsd_82        2119
+#define SYS_NAME_ID_i386_fbsd_90        2120
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -210,34 +223,36 @@
 #define SYS_NAME_ID_sparc_nbsd15        2503
 #define SYS_NAME_ID_sparc64_nbsd15      2504
 
-#define SYS_NAME_ID_i386_nbsd16		2510
-#define SYS_NAME_ID_alpha_nbsd16	2511
-#define SYS_NAME_ID_sparc_nbsd16	2512
-#define SYS_NAME_ID_sparc64_nbsd16	2513
-#define SYS_NAME_ID_macppc_nbsd16	2514
+#define SYS_NAME_ID_i386_nbsd16         2510
+#define SYS_NAME_ID_alpha_nbsd16        2511
+#define SYS_NAME_ID_sparc_nbsd16        2512
+#define SYS_NAME_ID_sparc64_nbsd16      2513
+#define SYS_NAME_ID_macppc_nbsd16       2514
 
-#define SYS_NAME_ID_i386_nbsd20		2520
-#define SYS_NAME_ID_alpha_nbsd20	2521
-#define SYS_NAME_ID_sparc_nbsd20	2522
-#define SYS_NAME_ID_sparc64_nbsd20	2523
-#define SYS_NAME_ID_macppc_nbsd20	2524
-#define SYS_NAME_ID_i386_nbsd21		2525
-#define SYS_NAME_ID_i386_nbsd30		2526
-#define SYS_NAME_ID_amd64_nbsd20	2527
-#define SYS_NAME_ID_i386_nbsd40 2528
-#define SYS_NAME_ID_i386_nbsd50 2529
-#define SYS_NAME_ID_sparc64_nbsd30 2530
-#define SYS_NAME_ID_sparc64_nbsd40 2531
-#define SYS_NAME_ID_sparc64_nbsd50 2532
-#define SYS_NAME_ID_amd64_nbsd30 2533
-#define SYS_NAME_ID_amd64_nbsd40 2534
-#define SYS_NAME_ID_amd64_nbsd50 2535
-#define SYS_NAME_ID_alpha_nbsd30 2536
-#define SYS_NAME_ID_alpha_nbsd40 2537
-#define SYS_NAME_ID_alpha_nbsd50 2538
-#define SYS_NAME_ID_macppc_nbsd30 2539
-#define SYS_NAME_ID_macppc_nbsd40 2540
-#define SYS_NAME_ID_macppc_nbsd50 2541
+#define SYS_NAME_ID_i386_nbsd20         2520
+#define SYS_NAME_ID_alpha_nbsd20        2521
+#define SYS_NAME_ID_sparc_nbsd20        2522
+#define SYS_NAME_ID_sparc64_nbsd20      2523
+#define SYS_NAME_ID_macppc_nbsd20       2524
+#define SYS_NAME_ID_i386_nbsd21         2525
+#define SYS_NAME_ID_i386_nbsd30         2526
+#define SYS_NAME_ID_amd64_nbsd20        2527
+#define SYS_NAME_ID_i386_nbsd40         2528
+#define SYS_NAME_ID_i386_nbsd50         2529
+#define SYS_NAME_ID_sparc64_nbsd30      2530
+#define SYS_NAME_ID_sparc64_nbsd40      2531
+#define SYS_NAME_ID_sparc64_nbsd50      2532
+#define SYS_NAME_ID_amd64_nbsd30        2533
+#define SYS_NAME_ID_amd64_nbsd40        2534
+#define SYS_NAME_ID_amd64_nbsd50        2535
+#define SYS_NAME_ID_alpha_nbsd30        2536
+#define SYS_NAME_ID_alpha_nbsd40        2537
+#define SYS_NAME_ID_alpha_nbsd50        2538
+#define SYS_NAME_ID_macppc_nbsd30       2539
+#define SYS_NAME_ID_macppc_nbsd40       2540
+#define SYS_NAME_ID_macppc_nbsd50       2541
+#define SYS_NAME_ID_amd64_nbsd60        2542
+#define SYS_NAME_ID_i386_nbsd60         2543
 
 #define SYS_NAME_ID_i386_obsd31		2600
 #define SYS_NAME_ID_i386_obsd32		2601
@@ -254,6 +269,9 @@
 #define SYS_NAME_ID_i386_obsd43         2612
 #define SYS_NAME_ID_i386_obsd44         2613
 #define SYS_NAME_ID_i386_obsd45         2614
+#define SYS_NAME_ID_i386_obsd46         2615
+#define SYS_NAME_ID_i386_obsd47         2616
+#define SYS_NAME_ID_i386_obsd48         2617
 
 #define SYS_NAME_ID_amd64_linux2        2700
 #define SYS_NAME_ID_amd64_linux22       2701
@@ -270,10 +288,16 @@
 #define SYS_NAME_ID_ppc64_linux24	2902
 #define SYS_NAME_ID_ppc64_linux26	2903
 
-#define SYS_NAME_ID_amd64_fbsd_53        3008
-#define SYS_NAME_ID_amd64_fbsd_70        3009
-#define SYS_NAME_ID_amd64_fbsd_71        3010
-#define SYS_NAME_ID_amd64_fbsd_80        3011
+#define SYS_NAME_ID_amd64_fbsd_53       3008
+#define SYS_NAME_ID_amd64_fbsd_70       3009
+#define SYS_NAME_ID_amd64_fbsd_71       3010
+#define SYS_NAME_ID_amd64_fbsd_72       3030
+#define SYS_NAME_ID_amd64_fbsd_73       3031
+#define SYS_NAME_ID_amd64_fbsd_74       3032
+#define SYS_NAME_ID_amd64_fbsd_80       3011
+#define SYS_NAME_ID_amd64_fbsd_81       3012
+#define SYS_NAME_ID_amd64_fbsd_82       3013
+#define SYS_NAME_ID_amd64_fbsd_90       3020
 
 #define SYS_NAME_ID_amd64_w2k           3400
 
@@ -286,6 +310,10 @@
 
 #define SYS_NAME_ID_i386_dfbsd_22        3900
 #define SYS_NAME_ID_i386_dfbsd_23        3901
+#define SYS_NAME_ID_i386_dfbsd_24        3902
+#define SYS_NAME_ID_i386_dfbsd_25        3903
+#define SYS_NAME_ID_i386_dfbsd_26        3904
+#define SYS_NAME_ID_i386_dfbsd_27        3905
 
 #define SYS_NAME_ID_amd64_obsd36        4005
 #define SYS_NAME_ID_amd64_obsd37        4006
@@ -297,10 +325,12 @@
 #define SYS_NAME_ID_amd64_obsd43        4012
 #define SYS_NAME_ID_amd64_obsd44        4013
 #define SYS_NAME_ID_amd64_obsd45        4014
-#define SYS_NAME_ID_amd64_obsd45        4014
+#define SYS_NAME_ID_amd64_obsd46        4015
+#define SYS_NAME_ID_amd64_obsd47        4016
+#define SYS_NAME_ID_amd64_obsd48        4017
 
 /*
- * Placeholder to keep system-wide standard flags since this file is included by all 
+ * Placeholder to keep system-wide standard flags since this file is included by all
  * files (i.e in afs/param.h)
  */
 #ifdef	notdef
