@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,29 +14,29 @@
  *                             to groups. This is an error in normal AFS.
  *                          2. If adding a group to a group call AddToSGEntry
  *                             to add the id of the group it's a member of.
- *                      
+ *
  *                      (4) function Delete
  *
  *                          1. Print a messsage if an error is returned from
- *                             FindByID() and PTDEBUG is defined. 
- *                          2. If removing a group from a group call   
- *                             RemoveFromSGEntry to remove the id of the  
- *                             group it's a member of.            
+ *                             FindByID() and PTDEBUG is defined.
+ *                          2. If removing a group from a group call
+ *                             RemoveFromSGEntry to remove the id of the
+ *                             group it's a member of.
  *                          3. Remove supergroup continuation records.
  *
- *                      (5) function RemoveFromGroup 
- *               
+ *                      (5) function RemoveFromGroup
+ *
  *                          1. Eliminate the code that tests for adding groups
- *                             to groups. This is an error in normal AFS. 
- *                          2. If removing a group from a group call 
- *                             RemoveFromSGEntry to remove the id of the 
+ *                             to groups. This is an error in normal AFS.
+ *                          2. If removing a group from a group call
+ *                             RemoveFromSGEntry to remove the id of the
  *                             group it's a member of.
  *
- *                      (6) Add new functions PR_ListSuperGroups and 
+ *                      (6) Add new functions PR_ListSuperGroups and
  *                          listSuperGroups.
- *                      
+ *
  *                      (7) function isAMemberOf
- *                      
+ *
  *                          1. Allow groups to be members of groups.
  *
  *                      Transarc does not currently use opcodes past 520, but
@@ -77,89 +77,84 @@
 #include <krb.h>
 #endif
 
-
-#define	IP_WILDCARDS	1	/* XXX Should be defined outside of here XXX */
-
 extern int restricted;
 extern struct ubik_dbase *dbase;
 extern int pr_noAuth;
-extern afs_int32 initd;
 extern char *pr_realmName;
 extern int prp_group_default;
 extern int prp_user_default;
 
-static afs_int32 iNewEntry(struct rx_call *call, char aname[], afs_int32 aid, 
+static afs_int32 iNewEntry(struct rx_call *call, char aname[], afs_int32 aid,
 			   afs_int32 oid, afs_int32 *cid);
-static afs_int32 newEntry(struct rx_call *call, char aname[], afs_int32 flag, 
-			  afs_int32 oid, afs_int32 *aid, afs_int32 *cid);       
+static afs_int32 newEntry(struct rx_call *call, char aname[], afs_int32 flag,
+			  afs_int32 oid, afs_int32 *aid, afs_int32 *cid);
 static afs_int32 whereIsIt(struct rx_call *call, afs_int32 aid, afs_int32 *apos,
 			   afs_int32 *cid);
-static afs_int32 dumpEntry(struct rx_call *call, afs_int32 apos, 
+static afs_int32 dumpEntry(struct rx_call *call, afs_int32 apos,
 			   struct prdebugentry *aentry, afs_int32 *cid);
-static afs_int32 addToGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid, 
+static afs_int32 addToGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid,
 			    afs_int32 *cid);
 static afs_int32 nameToID(struct rx_call *call, namelist *aname, idlist *aid);
 static afs_int32 idToName(struct rx_call *call, idlist *aid, namelist *aname);
 static afs_int32 Delete(struct rx_call *call, afs_int32 aid, afs_int32 *cid);
-static afs_int32 UpdateEntry(struct rx_call *call, afs_int32 aid, char *name, 
+static afs_int32 UpdateEntry(struct rx_call *call, afs_int32 aid, char *name,
 			     struct PrUpdateEntry *uentry, afs_int32 *cid);
-static afs_int32 removeFromGroup(struct rx_call *call, afs_int32 aid, 
+static afs_int32 removeFromGroup(struct rx_call *call, afs_int32 aid,
 				 afs_int32 gid, afs_int32 *cid);
-static afs_int32 getCPS(struct rx_call *call, afs_int32 aid, prlist *alist, 
+static afs_int32 getCPS(struct rx_call *call, afs_int32 aid, prlist *alist,
 			afs_int32 *over, afs_int32 *cid);
-static afs_int32 getCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost,
+static afs_int32 getCPS2(struct rx_call *call, afs_int32 aid, afs_uint32 ahost,
 			 prlist *alist, afs_int32 *over, afs_int32 *cid);
-static afs_int32 getHostCPS(struct rx_call *call, afs_int32 ahost, 
+static afs_int32 getHostCPS(struct rx_call *call, afs_uint32 ahost,
 			    prlist *alist, afs_int32 *over);
 static afs_int32 listMax(struct rx_call *call, afs_int32 *uid, afs_int32 *gid);
-static afs_int32 setMax(struct rx_call *call, afs_int32 aid, afs_int32 gflag, 
+static afs_int32 setMax(struct rx_call *call, afs_int32 aid, afs_int32 gflag,
 			afs_int32 *cid);
-static afs_int32 listEntry(struct rx_call *call, afs_int32 aid, 
+static afs_int32 listEntry(struct rx_call *call, afs_int32 aid,
 			   struct prcheckentry *aentry, afs_int32 *cid);
-static afs_int32 listEntries(struct rx_call *call, afs_int32 flag, 
-			     afs_int32 startindex, prentries *bulkentries, 
+static afs_int32 listEntries(struct rx_call *call, afs_int32 flag,
+			     afs_int32 startindex, prentries *bulkentries,
 			     afs_int32 *nextstartindex, afs_int32 *cid);
 static afs_int32 put_prentries(struct prentry *tentry, prentries *bulkentries);
-static afs_int32 changeEntry(struct rx_call *call, afs_int32 aid, char *name, 
+static afs_int32 changeEntry(struct rx_call *call, afs_int32 aid, char *name,
 			     afs_int32 oid, afs_int32 newid, afs_int32 *cid);
-static afs_int32 setFieldsEntry(struct rx_call *call, afs_int32 id, 
-				afs_int32 mask, afs_int32 flags, 
-				afs_int32 ngroups, afs_int32 nusers, 
-				afs_int32 spare1, afs_int32 spare2, 
+static afs_int32 setFieldsEntry(struct rx_call *call, afs_int32 id,
+				afs_int32 mask, afs_int32 flags,
+				afs_int32 ngroups, afs_int32 nusers,
+				afs_int32 spare1, afs_int32 spare2,
 				afs_int32 *cid);
 static afs_int32 listElements(struct rx_call *call, afs_int32 aid,
 			      prlist *alist, afs_int32 *over, afs_int32 *cid);
 #if defined(SUPERGROUPS)
-static afs_int32 listSuperGroups(struct rx_call *call, afs_int32 aid, 
-				 prlist *alist, afs_int32 *over, 
+static afs_int32 listSuperGroups(struct rx_call *call, afs_int32 aid,
+				 prlist *alist, afs_int32 *over,
 				 afs_int32 *cid);
 #endif
 static afs_int32 listOwned(struct rx_call *call, afs_int32 aid, prlist *alist,
 			   afs_int32 *lastP, afs_int32 *cid);
 static afs_int32 isAMemberOf(struct rx_call *call, afs_int32 uid, afs_int32 gid,
 			     afs_int32 *flag, afs_int32 *cid);
-#if IP_WILDCARDS
-static afs_int32 addWildCards(struct ubik_trans *tt, prlist *alist, 
-			      afs_int32 host);
-#endif
-static afs_int32 WhoIsThisWithName(struct rx_call *acall, 
-				   struct ubik_trans *at, afs_int32 *aid, 
+static afs_int32 addWildCards(struct ubik_trans *tt, prlist *alist,
+			      afs_uint32 host);
+static afs_int32 WhoIsThisWithName(struct rx_call *acall,
+				   struct ubik_trans *at, afs_int32 *aid,
 				   char *aname);
 
-/* When abort, reset initd so that the header is read in on next call.
+/* when we abort, the ubik cachedVersion will be reset, so we'll read in the
+ * header on the next call.
  * Abort the transaction and return the code.
  */
-#define ABORT_WITH(tt,code) return(initd=0,ubik_AbortTrans(tt),code)
+#define ABORT_WITH(tt,code) return(ubik_AbortTrans(tt),code)
 
 static int
-CreateOK(struct ubik_trans *ut, afs_int32 cid, afs_int32 oid, afs_int32 flag, 
+CreateOK(struct ubik_trans *ut, afs_int32 cid, afs_int32 oid, afs_int32 flag,
 	 int admin)
 {
-    if (restricted && !admin) 
+    if (restricted && !admin)
 	return 0;
 
     if (flag & PRFOREIGN) {
-	/* Foreign users are recognized by the '@' sign and 
+	/* Foreign users are recognized by the '@' sign and
 	 * not by the PRFOREIGN flag.
 	 */
 	return 0;
@@ -184,8 +179,8 @@ WhoIsThis(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid)
     int foreign = 0;
     /* aid is set to the identity of the caller, if known, else ANONYMOUSID */
     /* returns -1 and sets aid to ANONYMOUSID on any failure */
-    register struct rx_connection *tconn;
-    register afs_int32 code;
+    struct rx_connection *tconn;
+    afs_int32 code;
     char tcell[MAXKTCREALMLEN];
     char name[MAXKTCNAMELEN];
     char inst[MAXKTCNAMELEN];
@@ -252,13 +247,13 @@ SPR_INewEntry(struct rx_call *call, char aname[], afs_int32 aid, afs_int32 oid)
 }
 
 static afs_int32
-iNewEntry(struct rx_call *call, char aname[], afs_int32 aid, afs_int32 oid, 
+iNewEntry(struct rx_call *call, char aname[], afs_int32 aid, afs_int32 oid,
 	  afs_int32 *cid)
 {
     /* used primarily for conversion - not intended to be used as usual means
      * of entering people into the database. */
     struct ubik_trans *tt;
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 gflag = 0;
     int admin;
 
@@ -310,7 +305,7 @@ iNewEntry(struct rx_call *call, char aname[], afs_int32 aid, afs_int32 oid,
 
 
 afs_int32
-SPR_NewEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid, 
+SPR_NewEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid,
 	     afs_int32 *aid)
 {
     afs_int32 code;
@@ -324,10 +319,10 @@ SPR_NewEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid,
 }
 
 static afs_int32
-newEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid, 
+newEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid,
 	 afs_int32 *aid, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     int admin;
     char cname[PR_MAXNAMELEN];
@@ -346,7 +341,7 @@ newEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid,
 	ABORT_WITH(tt, code);
 
     /* this is for cross-cell self registration. It is not added in the
-     * SPR_INewEntry because we want self-registration to only do 
+     * SPR_INewEntry because we want self-registration to only do
      * automatic id assignment.
      */
     code = WhoIsThisWithName(call, tt, cid, cname);
@@ -389,7 +384,7 @@ SPR_WhereIsIt(struct rx_call *call, afs_int32 aid, afs_int32 *apos)
 static afs_int32
 whereIsIt(struct rx_call *call, afs_int32 aid, afs_int32 *apos, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
 
@@ -422,7 +417,7 @@ whereIsIt(struct rx_call *call, afs_int32 aid, afs_int32 *apos, afs_int32 *cid)
 
 
 afs_int32
-SPR_DumpEntry(struct rx_call *call, afs_int32 apos, 
+SPR_DumpEntry(struct rx_call *call, afs_int32 apos,
 	      struct prdebugentry *aentry)
 {
     afs_int32 code;
@@ -435,10 +430,10 @@ SPR_DumpEntry(struct rx_call *call, afs_int32 apos,
 }
 
 static afs_int32
-dumpEntry(struct rx_call *call, afs_int32 apos, struct prdebugentry *aentry, 
+dumpEntry(struct rx_call *call, afs_int32 apos, struct prdebugentry *aentry,
 	  afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
 
     code = Initdb();
@@ -495,7 +490,7 @@ SPR_AddToGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid)
 static afs_int32
 addToGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 tempu;
     afs_int32 tempg;
@@ -581,7 +576,7 @@ SPR_NameToID(struct rx_call *call, namelist *aname, idlist *aid)
 static afs_int32
 nameToID(struct rx_call *call, namelist *aname, idlist *aid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 i;
     int size;
@@ -630,13 +625,13 @@ nameToID(struct rx_call *call, namelist *aname, idlist *aid)
 
 	if (cell && afs_is_foreign_ticket_name(nameinst,NULL,cell,pr_realmName))
 	    code = NameToID(tt, aname->namelist_val[i], &aid->idlist_val[i]);
-	else 
+	else
 	    code = NameToID(tt, nameinst, &aid->idlist_val[i]);
 
 	if (code != PRSUCCESS)
 	    aid->idlist_val[i] = ANONYMOUSID;
         osi_audit(PTS_NmToIdEvent, code, AUD_STR,
-		   aname->namelist_val[i], AUD_ID, aid->idlist_val[i], 
+		   aname->namelist_val[i], AUD_ID, aid->idlist_val[i],
 		   AUD_END);
 	ViceLog(125, ("PTS_NameToID: code %d aname %s aid %d\n", code,
 		      aname->namelist_val[i], aid->idlist_val[i]));
@@ -674,7 +669,7 @@ SPR_IDToName(struct rx_call *call, idlist *aid, namelist *aname)
 static afs_int32
 idToName(struct rx_call *call, idlist *aid, namelist *aname)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 i;
     int size;
@@ -746,7 +741,7 @@ SPR_Delete(struct rx_call *call, afs_int32 aid)
 static afs_int32
 Delete(struct rx_call *call, afs_int32 aid, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     struct prentry tentry;
     afs_int32 loc, nptr;
@@ -979,7 +974,7 @@ Delete(struct rx_call *call, afs_int32 aid, afs_int32 *cid)
 }
 
 afs_int32
-SPR_UpdateEntry(struct rx_call *call, afs_int32 aid, char *name, 
+SPR_UpdateEntry(struct rx_call *call, afs_int32 aid, char *name,
 	        struct PrUpdateEntry *uentry)
 {
     afs_int32 code;
@@ -992,10 +987,10 @@ SPR_UpdateEntry(struct rx_call *call, afs_int32 aid, char *name,
 }
 
 afs_int32
-UpdateEntry(struct rx_call *call, afs_int32 aid, char *name, 
+UpdateEntry(struct rx_call *call, afs_int32 aid, char *name,
 	    struct PrUpdateEntry *uentry, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     struct prentry tentry;
     afs_int32 loc;
@@ -1083,10 +1078,10 @@ SPR_RemoveFromGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid)
 }
 
 static afs_int32
-removeFromGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid, 
+removeFromGroup(struct rx_call *call, afs_int32 aid, afs_int32 gid,
 		afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 tempu;
     afs_int32 tempg;
@@ -1165,10 +1160,10 @@ SPR_GetCPS(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *over)
 }
 
 static afs_int32
-getCPS(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *over, 
+getCPS(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *over,
        afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
     struct prentry tentry;
@@ -1210,7 +1205,6 @@ getCPS(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *over,
 }
 
 
-#ifdef IP_WILDCARDS
 int
 inCPS(prlist CPS, afs_int32 id)
 {
@@ -1222,28 +1216,27 @@ inCPS(prlist CPS, afs_int32 id)
     }
     return (0);
 }
-#endif /* IP_WILDCARDS */
 
 
 afs_int32
-SPR_GetCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost, 
+SPR_GetCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost,
 	    prlist *alist, afs_int32 *over)
 {
     afs_int32 code;
     afs_int32 cid = ANONYMOUSID;
 
     code = getCPS2(call, aid, ahost, alist, over, &cid);
-    osi_auditU(call, PTS_GetCPS2Event, code, AUD_ID, aid, AUD_HOST, ahost,
+    osi_auditU(call, PTS_GetCPS2Event, code, AUD_ID, aid, AUD_HOST, htonl(ahost),
 	       AUD_END);
     ViceLog(125, ("PTS_GetCPS2: code %d cid %d aid %d ahost %d\n", code, cid, aid, ahost));
     return code;
 }
 
 static afs_int32
-getCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost, prlist *alist, 
+getCPS2(struct rx_call *call, afs_int32 aid, afs_uint32 ahost, prlist *alist,
 	afs_int32 *over, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
     struct prentry tentry;
@@ -1299,10 +1292,8 @@ getCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost, prlist *alist,
 	code = GetList2(tt, &tentry, &host_tentry, alist, 1);
     else
 	code = GetList(tt, &tentry, alist, 1);
-#if IP_WILDCARDS
     if (!code)
 	code = addWildCards(tt, alist, ntohl(ahost));
-#endif /* IP_WILDCARDS */
     if (code != PRSUCCESS)
 	ABORT_WITH(tt, code);
 
@@ -1312,22 +1303,22 @@ getCPS2(struct rx_call *call, afs_int32 aid, afs_int32 ahost, prlist *alist,
 
 
 afs_int32
-SPR_GetHostCPS(struct rx_call *call, afs_int32 ahost, prlist *alist, 
+SPR_GetHostCPS(struct rx_call *call, afs_int32 ahost, prlist *alist,
 	       afs_int32 *over)
 {
     afs_int32 code;
 
     code = getHostCPS(call, ahost, alist, over);
-    osi_auditU(call, PTS_GetHCPSEvent, code, AUD_HOST, ahost, AUD_END);
+    osi_auditU(call, PTS_GetHCPSEvent, code, AUD_HOST, htonl(ahost), AUD_END);
     ViceLog(125, ("PTS_GetHostCPS: code %d ahost %d\n", code, ahost));
     return code;
 }
 
 afs_int32
-getHostCPS(struct rx_call *call, afs_int32 ahost, prlist *alist, 
+getHostCPS(struct rx_call *call, afs_uint32 ahost, prlist *alist,
 	   afs_int32 *over)
 {
-    register afs_int32 code, temp;
+    afs_int32 code, temp;
     struct ubik_trans *tt;
     struct prentry host_tentry;
     afs_int32 hostid;
@@ -1364,9 +1355,7 @@ getHostCPS(struct rx_call *call, afs_int32 ahost, prlist *alist,
 	} else
 	    fprintf(stderr, "FindByID Failed -- Not found\n");
     }
-#if IP_WILDCARDS
     code = addWildCards(tt, alist, ntohl(ahost));
-#endif /* IP_WILDCARDS */
   bad:
     if (code != PRSUCCESS)
 	ABORT_WITH(tt, code);
@@ -1390,7 +1379,7 @@ SPR_ListMax(struct rx_call *call, afs_int32 *uid, afs_int32 *gid)
 afs_int32
 listMax(struct rx_call *call, afs_int32 *uid, afs_int32 *gid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
 
     code = Initdb();
@@ -1432,7 +1421,7 @@ SPR_SetMax(struct rx_call *call, afs_int32 aid, afs_int32 gflag)
 static afs_int32
 setMax(struct rx_call *call, afs_int32 aid, afs_int32 gflag, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
 
     code = Initdb();
@@ -1479,10 +1468,10 @@ SPR_ListEntry(struct rx_call *call, afs_int32 aid, struct prcheckentry *aentry)
 }
 
 static afs_int32
-listEntry(struct rx_call *call, afs_int32 aid, struct prcheckentry *aentry, 
+listEntry(struct rx_call *call, afs_int32 aid, struct prcheckentry *aentry,
 	  afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
     struct prentry tentry;
@@ -1534,7 +1523,7 @@ listEntry(struct rx_call *call, afs_int32 aid, struct prcheckentry *aentry,
 }
 
 afs_int32
-SPR_ListEntries(struct rx_call *call, afs_int32 flag, afs_int32 startindex, 
+SPR_ListEntries(struct rx_call *call, afs_int32 flag, afs_int32 startindex,
 		prentries *bulkentries, afs_int32 *nextstartindex)
 {
     afs_int32 code;
@@ -1547,7 +1536,7 @@ SPR_ListEntries(struct rx_call *call, afs_int32 flag, afs_int32 startindex,
 }
 
 static afs_int32
-listEntries(struct rx_call *call, afs_int32 flag, afs_int32 startindex, 
+listEntries(struct rx_call *call, afs_int32 flag, afs_int32 startindex,
 	    prentries *bulkentries, afs_int32 *nextstartindex, afs_int32 *cid)
 {
     afs_int32 code;
@@ -1670,7 +1659,7 @@ put_prentries(struct prentry *tentry, prentries *bulkentries)
 }
 
 afs_int32
-SPR_ChangeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid, 
+SPR_ChangeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid,
 		afs_int32 newid)
 {
     afs_int32 code;
@@ -1684,10 +1673,10 @@ SPR_ChangeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid,
 }
 
 static afs_int32
-changeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid, 
+changeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid,
 	    afs_int32 newid, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 pos;
 
@@ -1729,10 +1718,10 @@ changeEntry(struct rx_call *call, afs_int32 aid, char *name, afs_int32 oid,
 }
 
 afs_int32
-SPR_SetFieldsEntry(struct rx_call *call, 
-		   afs_int32 id, 
-		   afs_int32 mask, /* specify which fields to update */ 
-		   afs_int32 flags, afs_int32 ngroups, afs_int32 nusers, 
+SPR_SetFieldsEntry(struct rx_call *call,
+		   afs_int32 id,
+		   afs_int32 mask, /* specify which fields to update */
+		   afs_int32 flags, afs_int32 ngroups, afs_int32 nusers,
 		   afs_int32 spare1, afs_int32 spare2)
 {
     afs_int32 code;
@@ -1747,13 +1736,13 @@ SPR_SetFieldsEntry(struct rx_call *call,
 }
 
 static afs_int32
-setFieldsEntry(struct rx_call *call, 
-	       afs_int32 id, 
-	       afs_int32 mask, /* specify which fields to update */ 
-	       afs_int32 flags, afs_int32 ngroups, afs_int32 nusers, 
+setFieldsEntry(struct rx_call *call,
+	       afs_int32 id,
+	       afs_int32 mask, /* specify which fields to update */
+	       afs_int32 flags, afs_int32 ngroups, afs_int32 nusers,
 	       afs_int32 spare1, afs_int32 spare2, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 pos;
     struct prentry tentry;
@@ -1831,7 +1820,7 @@ setFieldsEntry(struct rx_call *call,
 }
 
 afs_int32
-SPR_ListElements(struct rx_call *call, afs_int32 aid, prlist *alist, 
+SPR_ListElements(struct rx_call *call, afs_int32 aid, prlist *alist,
 		 afs_int32 *over)
 {
     afs_int32 code;
@@ -1844,10 +1833,10 @@ SPR_ListElements(struct rx_call *call, afs_int32 aid, prlist *alist,
 }
 
 static afs_int32
-listElements(struct rx_call *call, afs_int32 aid, prlist *alist, 
+listElements(struct rx_call *call, afs_int32 aid, prlist *alist,
 	     afs_int32 *over, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
     struct prentry tentry;
@@ -1892,7 +1881,7 @@ listElements(struct rx_call *call, afs_int32 aid, prlist *alist,
 
 
 afs_int32
-SPR_ListSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist, 
+SPR_ListSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist,
 		    afs_int32 *over)
 {
 #if defined(SUPERGROUPS)
@@ -1910,10 +1899,10 @@ SPR_ListSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist,
 
 #if defined(SUPERGROUPS)
 static afs_int32
-listSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist, 
+listSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist,
 		afs_int32 *over, afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     afs_int32 temp;
     struct prentry tentry;
@@ -1958,7 +1947,7 @@ listSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist,
 
 #endif /* SUPERGROUPS */
 
-/* 
+/*
  * SPR_ListOwned
  * List the entries owned by this id.  If the id is zero,
  * return the orphans list. This will return up to PR_MAXGROUPS
@@ -1966,7 +1955,7 @@ listSuperGroups(struct rx_call *call, afs_int32 aid, prlist *alist,
  * maximum value is enforced in GetOwnedChain().
  */
 afs_int32
-SPR_ListOwned(struct rx_call *call, afs_int32 aid, prlist *alist, 
+SPR_ListOwned(struct rx_call *call, afs_int32 aid, prlist *alist,
 	      afs_int32 *lastP)
 {
     afs_int32 code;
@@ -1979,10 +1968,10 @@ SPR_ListOwned(struct rx_call *call, afs_int32 aid, prlist *alist,
 }
 
 afs_int32
-listOwned(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *lastP, 
+listOwned(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *lastP,
 	  afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
     struct prentry tentry;
     afs_int32 head = 0;
@@ -2051,7 +2040,7 @@ listOwned(struct rx_call *call, afs_int32 aid, prlist *alist, afs_int32 *lastP,
 }
 
 afs_int32
-SPR_IsAMemberOf(struct rx_call *call, afs_int32 uid, afs_int32 gid, 
+SPR_IsAMemberOf(struct rx_call *call, afs_int32 uid, afs_int32 gid,
 		afs_int32 *flag)
 {
     afs_int32 code;
@@ -2068,7 +2057,7 @@ static afs_int32
 isAMemberOf(struct rx_call *call, afs_int32 uid, afs_int32 gid, afs_int32 *flag,
 	    afs_int32 *cid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct ubik_trans *tt;
 
     code = Initdb();
@@ -2117,9 +2106,8 @@ isAMemberOf(struct rx_call *call, afs_int32 uid, afs_int32 gid, afs_int32 *flag,
     return code;
 }
 
-#if IP_WILDCARDS
 static afs_int32
-addWildCards(struct ubik_trans *tt, prlist *alist, afs_int32 host)
+addWildCards(struct ubik_trans *tt, prlist *alist, afs_uint32 host)
 {
     afs_int32 temp;
     struct prentry tentry;
@@ -2165,16 +2153,15 @@ addWildCards(struct ubik_trans *tt, prlist *alist, afs_int32 host)
 	qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return 0;
 }
-#endif /* IP_WILDCARDS */
 
 static afs_int32
-WhoIsThisWithName(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid, 
+WhoIsThisWithName(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid,
 		  char *aname)
 {
     /* aid is set to the identity of the caller, if known, else ANONYMOUSID */
     /* returns -1 and sets aid to ANONYMOUSID on any failure */
-    register struct rx_connection *tconn;
-    register afs_int32 code;
+    struct rx_connection *tconn;
+    afs_int32 code;
     char tcell[MAXKTCREALMLEN];
     char name[MAXKTCNAMELEN];
     char inst[MAXKTCNAMELEN];
