@@ -17,8 +17,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/test/listVicepx.c,v 1.7 2006/03/17 19:54:54 shadow Exp $");
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
@@ -99,7 +97,7 @@ RCSID
 #include "volume.h"
 #include "vldb.h"
 #include "partition.h"
-#include "afs/assert.h"
+#include "afs/afs_assert.h"
 #include "filesignal.h"
 #include "vutils.h"
 #include "daemon_com.h"
@@ -546,9 +544,9 @@ printDirs(partitionName)
 ader */
 static
 ComputeUsedPages(dhp)
-     register struct DirHeader *dhp;
+     struct DirHeader *dhp;
 {
-    register afs_int32 usedPages, i;
+    afs_int32 usedPages, i;
 
     if (dhp->header.pgcount != 0) {
 	/* new style */
